@@ -287,13 +287,16 @@ instance (FitsInByte (SumArity (a :+: b)), GStorePeekSum 0 (a :+: b))
 type FitsInByte n = FitsInByteResult (n <=? 255)
 
 type family FitsInByteResult (b :: Bool) :: Constraint where
-    FitsInByteResult True = ()
-    FitsInByteResult False = TypeErrorMessage
+    FitsInByteResult 'True = ()
+    FitsInByteResult 'False = TypeErrorMessage
         "Generic deriving of Store instances can only be used on datatypes with fewer than 256 constructors."
 
 type family TypeErrorMessage (a :: Symbol) :: Constraint where
 #if MIN_VERSION_base(4,9,0)
-    TypeErrorMessage a = TypeError (Text a)
+    TypeErrorMessage a = TypeError ('Text a)
+-- GHC < 8.0 does not support empty closed type families
+#elif __GLASGOW_HASKELL__ < 800
+    TypeErrorMessage a = a ~ ""
 #endif
 
 -- Similarly to splitting up the generic class into multiple classes, we
