@@ -76,10 +76,12 @@ import qualified Data.ByteString.Internal as BS
 import qualified Data.ByteString.Lazy as LBS
 import qualified Data.ByteString.Short.Internal as SBS
 import           Data.Containers (IsMap, ContainerKey, MapValue, mapFromList, mapToList, IsSet, setFromList)
+import           Data.Complex (Complex (..))
 import           Data.Data (Data)
 import           Data.Fixed (Fixed (..), Pico)
 import           Data.Foldable (forM_, foldl')
 import           Data.Functor.Contravariant
+import           Data.Functor.Identity (Identity (..))
 import           Data.HashMap.Strict (HashMap)
 import           Data.HashSet (HashSet)
 import           Data.Hashable (Hashable)
@@ -118,10 +120,15 @@ import           Foreign.C.Types ()
 import           Foreign.Ptr (plusPtr, minusPtr)
 import           Foreign.Storable (Storable, sizeOf)
 import           GHC.Generics (Generic)
+#ifdef INTEGER_GMP
 import qualified GHC.Integer.GMP.Internals as I
+import           GHC.Types (Int (I#))
+#else
+import           GHC.Types (Word (W#))
+import qualified GHC.Integer.Simple.Internals as I
+#endif
 import           GHC.Real (Ratio(..))
 import           GHC.TypeLits
-import           GHC.Types (Int (I#))
 import           Instances.TH.Lift ()
 import           Language.Haskell.TH
 import           Language.Haskell.TH.Instances ()
@@ -132,9 +139,11 @@ import           Prelude
 import           TH.Derive
 
 -- Conditional import to avoid warning
+#ifdef INTEGER_GMP
 #if MIN_VERSION_integer_gmp(1,0,0)
 import           GHC.Prim (sizeofByteArray#)
 #endif
+#endif
 
 ------------------------------------------------------------------------
 -- Utilities for defining list-like 'Store' instances in terms of 'IsSequence'
@@ -390,6 +399,22 @@ instance Store BS.ByteString where
         fp <- peekToPlainForeignPtr "Data.ByteString.ByteString" len
         return (BS.PS fp 0 len)
 
+#if MIN_VERSION_template_haskell(2,16,0)
+-- | Template Haskell Bytes are nearly identical to ByteString, but it
+-- can't depend on ByteString.
+instance Store Bytes where
+    size = VarSize $ \x ->
+        sizeOf (undefined :: Int) +
+        fromIntegral (bytesSize x)
+    poke (Bytes sourceFp sourceOffset sourceLength) = do
+        poke sourceLength
+        pokeFromForeignPtr sourceFp (fromIntegral sourceOffset) (fromIntegral sourceLength)
+    peek = do
+        len <- peek
+        fp <- peekToPlainForeignPtr "Data.ByteString.ByteString" (fromIntegral len)
+        return (Bytes fp 0 len)
+#endif
+
 instance Store SBS.ShortByteString where
     size = VarSize $ \x ->
          sizeOf (undefined :: Int) +
@@ -573,6 +598,7 @@ peekArray = do
 {-# INLINE peekArray #-}
 
 instance Store Integer where
+#ifdef INTEGER_GMP
 #if MIN_VERSION_integer_gmp(1,0,0)
     size = VarSize $ \ x ->
         sizeOf (undefined :: Word8) + case x of
@@ -640,6 +666,51 @@ instance Store Integer where
           when (r /= 0) (peekException "Buffer size stored for encoded Integer not divisible by Word size (to get limb count).")
           return (I.J# sz arr)
 #endif
+#else
+    -- NOTE: integer-simple uses a different encoding than GMP
+    size = VarSize $ \ x ->
+        sizeOf (undefined :: Word8) + case x of
+            I.Positive ds -> (1 + fromIntegral (numDigits ds)) * sizeOf (undefined :: Word)
+            I.Negative ds -> (1 + fromIntegral (numDigits ds)) * sizeOf (undefined :: Word)
+            I.Naught -> 0
+      where
+    poke x = case x of
+      I.Naught -> poke (0 :: Word8)
+      I.Positive ds -> do
+        poke (1 :: Word8)
+        poke (numDigits ds)
+        pokeDigits ds
+      I.Negative ds -> do
+        poke (2 :: Word8)
+        poke (numDigits ds)
+        pokeDigits ds
+      where
+        pokeDigits I.None = pure ()
+        pokeDigits (I.Some d ds) = poke (W# d) *> pokeDigits ds
+    peek = do
+      tag <- peek :: Peek Word8
+      case tag of
+        0 -> pure I.Naught
+        1 -> do
+          len <- peek :: Peek Word
+          I.Positive <$> peekDigits len
+        2 -> do
+          len <- peek :: Peek Word
+          I.Negative <$> peekDigits len
+        _ -> peekException "Invalid Integer tag"
+      where
+        peekDigits i
+          | i <= 0 = pure I.None
+          | otherwise = do
+              W# d <- peek
+              ds <- peekDigits (i - 1)
+              pure $! I.Some d ds
+
+numDigits :: I.Digits -> Word
+numDigits = go 0
+  where go !acc I.None = acc
+        go !acc (I.Some _ ds) = go (acc + 1) ds
+#endif
 
 -- instance Store GHC.Fingerprint.Types.Fingerprint where
 
@@ -666,6 +737,19 @@ instance Store a => Store (Ratio a) where
     poke (x :% y) = poke (x, y)
     peek = uncurry (:%) <$> peek
 
+-- Similarly, manual implementation due to no Generic instance for
+-- Complex and Identity in GHC-7.10 and earlier.
+
+instance Store a => Store (Complex a) where
+    size = combineSize (\(x :+ _) -> x) (\(_ :+ y) -> y)
+    poke (x :+ y) = poke (x, y)
+    peek = uncurry (:+) <$> peek
+
+instance Store a => Store (Identity a) where
+    size = contramap (\(Identity x) -> x) size
+    poke (Identity x) = poke x
+    peek = Identity <$> peek
+
 instance Store Time.Day where
     size = contramap Time.toModifiedJulianDay (size :: Size Integer)
     poke = poke . Time.toModifiedJulianDay
@@ -688,6 +772,7 @@ instance Store a => Store (Product a)
 instance Store a => Store (First a)
 instance Store a => Store (Last a)
 instance Store a => Store (Maybe a)
+instance Store a => Store (Const a b)
 
 -- FIXME: have TH deriving handle unboxed fields?
 
