@@ -24,11 +24,11 @@ module Data.Store.TH.Internal
 
 import           Control.Applicative
 import           Data.Complex ()
-import           Data.Generics.Aliases (extT)
-import           Data.Generics.Schemes (listify, everywhere)
-import           Data.List (find, nub)
+import           Data.Generics.Aliases (extT, mkQ, extQ)
+import           Data.Generics.Schemes (listify, everywhere, something)
+import           Data.List (find)
 import qualified Data.Map as M
-import           Data.Maybe (fromMaybe)
+import           Data.Maybe (fromMaybe, isJust)
 import           Data.Primitive.ByteArray
 import           Data.Primitive.Types
 import           Data.Store.Core
@@ -287,13 +287,24 @@ deriveManyStoreFromStorable p = do
         \(TypeclassInstance cs ty _) ->
         let argTy = head (tail (unAppsT ty))
             tyNameLit = LitE (StringL (pprint ty)) in
-        if p argTy
+        if p argTy && not (superclassHasStorable cs)
             then Just $ makeStoreInstance cs argTy
                 (AppE (VarE 'sizeStorableTy) tyNameLit)
                 (AppE (VarE 'peekStorableTy) tyNameLit)
                 (VarE 'pokeStorable)
             else Nothing
 
+-- See #143. Often Storable superclass constraints should instead be
+-- Store constraints, so instead it just warns for these.
+superclassHasStorable :: Cxt -> Bool
+superclassHasStorable = isJust . something (mkQ Nothing justStorable `extQ` ignoreStrings)
+  where
+    justStorable :: Type -> Maybe ()
+    justStorable (ConT n) | n == ''Storable = Just ()
+    justStorable _ = Nothing
+    ignoreStrings :: String -> Maybe ()
+    ignoreStrings _ = Nothing
+
 ------------------------------------------------------------------------
 -- Vector
 
@@ -336,24 +347,32 @@ deriveManyStoreUnboxVector :: Q [Dec]
 deriveManyStoreUnboxVector = do
     unboxes <- getUnboxInfo
     stores <- postprocess . instancesMap <$> getInstances ''Store
-    let unboxInsts =
+    unboxInstances <- postprocess . instancesMap <$> getInstances ''UV.Unbox
+    let dataFamilyDecls =
             M.fromList (map (\(preds, ty, cons) -> ([AppT (ConT ''UV.Vector) ty], (preds, cons))) unboxes)
             `M.difference`
             stores
+#if MIN_VERSION_template_haskell(2,10,0)
+        substituteConstraint (AppT (ConT n) arg)
+          | n == ''UV.Unbox = AppT (ConT ''Store) (AppT (ConT ''UV.Vector) arg)
+#else
+        substituteConstraint (ClassP n [arg])
+          | n == ''UV.Unbox = ClassP ''Store [AppT (ConT ''UV.Vector) arg]
+#endif
+        substituteConstraint x = x
     -- TODO: ideally this would use a variant of 'deriveStore' which
     -- assumes VarSize.
-    forM (M.toList unboxInsts) $ \case
-        ([ty], (preds, cons)) -> do
-            {-
-            -- While this approach is reasonable-ish, it ends up
-            -- requiring UndecidableInstances.
-            let extraPreds =
-                    map (AppT (ConT ''Store)) $
-                    filter (not . isMonoType) $
-                    concatMap (map snd . dcFields) cons
-            -}
-            let extraPreds = map (storePred . AppT (ConT ''UV.Vector)) $ listify isVarT ty
-            deriveStore (nub (preds ++ extraPreds)) ty cons
+    forM (M.toList dataFamilyDecls) $ \case
+        ([ty], (_, cons)) -> do
+            let headTy = getTyHead (unAppsT ty !! 1)
+            (preds, ty') <- case M.lookup [headTy] unboxInstances of
+              Nothing -> do
+                reportWarning $ "No Unbox instance found for " ++ pprint headTy
+                return ([], ty)
+              Just (TypeclassInstance cs (AppT _ ty') _) ->
+                return (map substituteConstraint cs, AppT (ConT ''UV.Vector) ty')
+              Just _ -> fail "Impossible case"
+            deriveStore preds ty' cons
         _ -> fail "impossible case in deriveManyStoreUnboxVector"
 
 -- TODO: Add something for this purpose to TH.ReifyDataType
@@ -363,7 +382,14 @@ getUnboxInfo = do
     FamilyI _ insts <- reify ''UV.Vector
     return (map (everywhere (id `extT` dequalVarT) . go) insts)
   where
-#if MIN_VERSION_template_haskell(2,11,0)
+#if MIN_VERSION_template_haskell(2,15,0)
+    go (NewtypeInstD preds _ lhs _ con _)
+      | [_, ty] <- unAppsT lhs
+      = (preds, ty, conToDataCons con)
+    go (DataInstD preds _ lhs _ cons _)
+      | [_, ty] <- unAppsT lhs
+      = (preds, ty, concatMap conToDataCons cons)
+#elif MIN_VERSION_template_haskell(2,11,0)
     go (NewtypeInstD preds _ [ty] _ con _) = (preds, ty, conToDataCons con)
     go (DataInstD preds _ [ty] _ cons _) = (preds, ty, concatMap conToDataCons cons)
 #else
@@ -371,9 +397,11 @@ getUnboxInfo = do
     go (DataInstD preds _ [ty] cons _) = (preds, ty, concatMap conToDataCons cons)
 #endif
     go x = error ("Unexpected result from reifying Unboxed Vector instances: " ++ pprint x)
+    dequalVarT :: Type -> Type
     dequalVarT (VarT n) = VarT (dequalify n)
     dequalVarT ty = ty
 
+
 ------------------------------------------------------------------------
 -- Utilities
 
