@@ -16,12 +16,14 @@ module Data.StoreSpec where
 import           Control.Applicative
 import           Control.Exception (evaluate)
 import           Control.Monad (unless)
+import           Control.Monad.Fail (MonadFail)
 import qualified Data.Array.Unboxed as A
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy as LBS
 import qualified Data.ByteString.Short as SBS
 import           Data.Complex (Complex(..))
 import           Data.Containers (mapFromList, setFromList)
+import           Data.Generics (listify)
 import           Data.HashMap.Strict (HashMap)
 import           Data.HashSet (HashSet)
 import           Data.Hashable (Hashable)
@@ -31,7 +33,9 @@ import           Data.IntSet (IntSet)
 import qualified Data.List.NonEmpty as NE
 import           Data.Map (Map)
 import           Data.Monoid
+#if !MIN_VERSION_primitive(0,7,0)
 import           Data.Primitive.Types (Addr)
+#endif
 import           Data.Proxy (Proxy(..))
 import           Data.Sequence (Seq)
 import           Data.Sequences (fromList)
@@ -67,6 +71,7 @@ import           System.Clock (TimeSpec)
 import           System.Posix.Types
 import           Test.Hspec hiding (runIO)
 import           Test.SmallCheck.Series
+import           TH.Utilities (unAppsT)
 
 ------------------------------------------------------------------------
 -- Instances for base types
@@ -133,15 +138,21 @@ $(do let ns = [ ''CUSeconds, ''CClock, ''CTime, ''CUChar, ''CSize, ''CSigAtomic
 $(do tys <- getAllInstanceTypes1 ''PV.Prim
      let f ty = [d| instance (Serial m $(return ty), Monad m) => Serial m (PV.Vector $(return ty)) where
                       series = fmap PV.fromList series |]
-     concat <$> mapM f tys)
+     concat <$> mapM f (filter (\ty -> length (unAppsT ty) == 1) tys))
 
--- Instance needed for generic TH instances
+{-
+-- Instances for TH are not currently needed, because the tests for
+-- serialization roundtripping are disabled due to Bytes' Eq instance,
+-- see issue #150.
 
 -- Needs to be done manually because in GHC 7.8's TH, NameFlavour uses
 -- unboxed values and cannot use generic deriving. So we skip having an
 -- instance for it.
 instance Monad m => Serial m Name where series = fmap mkName series
 
+instance Monad m => Serial m Bytes where
+  series = fmap ((\(BS.PS p o l) -> Bytes p (fromIntegral o) (fromIntegral l)) . BS.pack) series
+
 -- Serial instances for (Generic a) types.
 
 -- FIXME: generating for TH instances is probably just adding
@@ -153,11 +164,18 @@ $(do thNames <- reifyManyWithoutInstances
      let ns = [ ''Any, ''All ] ++ thNames
          f n = [d| instance Monad m => Serial m $(conT n) |]
      concat <$> mapM f ns)
+-}
 
 $(do let ns = [ ''Dual, ''Sum, ''Product, ''First, ''Last ]
          f n = [d| instance (Monad m, Serial m a) => Serial m ($(conT n) a) |]
      concat <$> mapM f ns)
 
+instance Monad m => Serial m Any where
+    series = fmap Any series
+
+instance Monad m => Serial m All where
+    series = fmap All series
+
 instance Monad m => Serial m Fingerprint where
     series = generate (\_ -> [Fingerprint 0 0, Fingerprint maxBound maxBound])
 
@@ -233,11 +251,6 @@ instance (Monad m, Serial m a) => Serial m (Tagged a)
 instance Monad m => Serial m Void where
     series = generate (\_ -> [])
 
-#if !MIN_VERSION_template_haskell(2,11,0)
-deriving instance Show NameFlavour
-deriving instance Show NameSpace
-#endif
-
 -- We define our own Serial instance for 'Ratio' because of <https://github.com/feuerbach/smallcheck/pull/34>
 
 newtype SerialRatio a = SerialRatio (Ratio a)
@@ -284,9 +297,11 @@ spec :: Spec
 spec = do
     describe "Store on all monomorphic instances"
         $(do insts <- getAllInstanceTypes1 ''Store
-             omitTys0 <- sequence
-                 [ [t| Addr |]
-                 , [t| CUIntPtr |]
+             omitTys0 <- sequence $
+#if !MIN_VERSION_primitive(0,7,0)
+                 [t| Addr |] :
+#endif
+                 [ [t| CUIntPtr |]
                  , [t| CIntPtr |]
                  , [t| IntPtr |]
                  , [t| WordPtr |]
@@ -299,11 +314,13 @@ spec = do
                  , [t| TimeSpec |]
                  ]
              omitTys <- (omitTys0 ++) <$> mapM (\ty -> [t| PV.Vector $(pure ty) |]) omitTys0
-             let f ty = isMonoType ty && ty `notElem` omitTys
+             let f ty = isMonoType ty && ty `notElem` omitTys && null (listify isThName ty)
                  filtered = filter f insts
+                 -- Roundtrip testing of TH instances is disabled - see issue #150
+                 isThName n = nameModule n == Just "Language.Haskell.TH.Syntax"
              smallcheckManyStore verbose 2 $ map return filtered)
     it "Store on non-numeric Float/Double values" $ do
-        let testNonNumeric :: forall a m. (RealFloat a, Eq a, Show a, Typeable a, Store a, Monad m) => Proxy a -> m ()
+        let testNonNumeric :: forall a m. (RealFloat a, Eq a, Show a, Typeable a, Store a, Monad m, MonadFail m) => Proxy a -> m ()
             testNonNumeric _proxy = do
                 assertRoundtrip verbose ((1/0) :: a)
                 assertRoundtrip verbose ((-1/0) :: a)
